from django.views import View
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import ListView
from django.contrib.auth.decorators import login_required, user_passes_test
from django.urls.base import reverse
from mri_users.views import AdministratorMixin, user_is_admin
from mri_links.models import LiensOutils
from mri_links.forms import ManageLiens
from django.forms.models import model_to_dict
from django.apps import apps
if apps.is_installed('mri_db_logger'):
    from mri_db_logger.models import ActionsLogs

class ListeLiensView(LoginRequiredMixin, AdministratorMixin, ListView):
    model = LiensOutils
    context_object_name = 'liste_liens'
    template_name = 'manage_liens_outils/liste_liens_outils.html'


class EditLiensOutils(LoginRequiredMixin, AdministratorMixin, View):
    """
    Edite ou ajoute un liens outils
    """
    template_name = 'manage_liens_outils/edit_lien_outils.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        if 'lien' in self.kwargs:
            lien = get_object_or_404(LiensOutils, pk=self.kwargs['lien'])
            manage_lien_form = ManageLiens(instance=lien)
        else:
            manage_lien_form = ManageLiens()
        return render(request, self.template_name, {
            'manage_lien_form': manage_lien_form})

    def post(self, request, *args, **kwargs):
        if 'lien' in self.kwargs:
            lien = get_object_or_404(LiensOutils, pk=self.kwargs['lien'])
            before = model_to_dict(lien)
            manage_lien_form = ManageLiens(request.POST, instance=lien)
            new = False
        else:
            manage_lien_form = ManageLiens(request.POST)
            new = True
        if manage_lien_form.is_valid():
            if apps.is_installed('mri_db_logger'):
                if new:
                    ActionsLogs.objects.create(action='Ajoute lien',
                                               infos={'texte_lien_fr': manage_lien_form.cleaned_data['texte_lien_fr']})
            manage_lien_form.save()
            if apps.is_installed('mri_db_logger') and not new:
                lien.refresh_from_db()
                after = model_to_dict(lien)
                ActionsLogs.objects.create(action='Edite lien',
                                           infos={'before': before,
                                                  'after': after})
            return redirect(reverse('manage_liens_outils'))
        return render(request, self.template_name, {
            'manage_lien_form': manage_lien_form})

@login_required
@user_passes_test(user_is_admin)
def efface_lien(request, lien_pk):
    """
    Supprime un lien appli
    """
    lien = get_object_or_404(LiensOutils, pk=lien_pk)
    if apps.is_installed('mri_db_logger'):
        infos = model_to_dict(lien)
        ActionsLogs.objects.create(action='Efface lien',
                                   infos=infos)
    lien.delete()
    return redirect(reverse('manage_liens_outils'))
